/*
 *  IniFile.java
 *  Handling of configuration files in windows ini format
 */
package de.pschuett.util;

import java.io.*;
import java.util.*;

/**
 *  Handling of configuration files im windows ini format. <br>
 *  It deals with sections and keys, e.g.:<br>
 *  <pre>
 *  [Section1]
 *  Key11=...
 *  Key12=...
 *
 *  [Section2]
 *  Key21=...
 *  Key22=...
 *  ...
 *  </pre>
 *
 *@author     Peter Schtt
 *@created    11. Dezember 2002
 */
public class IniFile
{
    private SortedMap data;
    private File mFile;


    /**
     *  Constructor
     *
     *@param  file                       Description of Parameter
     *@exception  IOException            Description of Exception
     *@exception  IniFileException       Description of Exception
     *@exception  FileNotFoundException  Description of Exception
     */
    public IniFile(File file) throws IOException, IniFileException, FileNotFoundException
    {
        mFile = file;
        data = new TreeMap();
        LineNumberReader linenumberreader = null;
        if (file.exists())
        {
            linenumberreader = new LineNumberReader(new FileReader(file));
        }
        Properties props = null;
        if (linenumberreader == null)
        {
            return;
        }
        String line;
        while ((line = linenumberreader.readLine()) != null)
        {
            if ((line = line.trim()).length() > 0)
            {
                if (isSection(line))
                {
                    props = new Properties();
                    data.put(parseSection(line), props);
                }
                else
                        if (isKey(line))
                {
                    parseKey(line, props);
                }
                else
                {
                    throw new IniFileException("Problem reading line:" + linenumberreader.getLineNumber());
                }
            }
        }
        linenumberreader.close();
    }


    /**
     *  isKey checks if a text line contains a key
     *
     *@param  line  line of the ini file
     *@return       The Key value
     */
    private static boolean isKey(String line)
    {
        int ii = line.indexOf("=");
        return ii > 0 && ii == line.lastIndexOf("=");
    }


    /**
     *  isSection checks if a text line contains a key
     *
     *@param  line  line of the ini file
     *@return       The Section value
     */
    private static boolean isSection(String line)
    {
        boolean flag;
        try
        {
            flag = line.charAt(0) == '[' && line.endsWith("]");
        }
        catch (StringIndexOutOfBoundsException _ex)
        {
            flag = false;
        }
        return flag;
    }


    /**
     *  parse key
     *
     *@param  key
     *@param  map  Description of Parameter
     *@return      Description of the Returned Value
     */
    private void parseKey(String key, Properties props)
    {
        try
        {
            int ii = key.indexOf("=");
            props.setProperty(key.substring(0, ii).trim(), key.substring(ii + 1).trim());
        }
        catch (StringIndexOutOfBoundsException _ex)
        {
        }
    }


    /**
     *  parseSection
     *
     *@param  section
     *@return          Description of the Returned Value
     */
    private static String parseSection(String section)
    {
        String s1;
        try
        {
            s1 = section.substring(1, section.length() - 1);
        }
        catch (StringIndexOutOfBoundsException _ex)
        {
            s1 = "";
        }
        return s1;
    }


    /**
     *  getSections returns all sections in this ini file
     *
     *@return    vector of section names
     */
    public Collection getSections()
    {
        return data.keySet();
    }


    /**
     *  getSection returns all sections in this ini file
     *
     *@param  section  Description of Parameter
     *@return          vector of section names
     */
    public Properties getSectionValues(String section)
    {
        return (Properties) data.get(section);
    }


    /**
     *  flush flushes ini data into mFilename
     *
     *@exception  IOException  Description of Exception
     */
    public void flush() throws IOException
    {
        writeFile(mFile);
    }


    /**
     *  writeFile writes ini data in a new file
     *
     *@param  file             Description of Parameter
     *@exception  IOException  Description of Exception
     */
    public void writeFile(File file) throws IOException
    {
        BufferedWriter bufferedwriter = null;
        bufferedwriter = new BufferedWriter(new FileWriter(file));
        printOn(bufferedwriter);
        bufferedwriter.close();
    }


    /**
     *  sectionExists proof if a section exists
     *
     *@param  section  Description of Parameter
     *@return          true if sections exists, false if not such a section
     *      exists
     */
    public boolean sectionExists(String section)
    {
        return data.containsKey(section);
    }


    /**
     *  deleteSection delete a section in this ini file
     *
     *@param  section  Description of Parameter
     *@return          true if sections exists, false if not such a section
     *      exists
     */
    public Object deleteSection(String section)
    {
        return data.remove(section);
    }


    /**
     *  deleteSection delete a section in this ini file
     *
     *@param  section  Description of Parameter
     *@return          true if sections exists, false if not such a section
     *      exists
     */
    public Object createSection(String section)
    {
        return data.put(section, new HashMap());
    }


    /**
     *  readString reads a value of a key in a section. If the section or key is
     *  unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue
     *@return               value of the key in this section
     */
    public String readString(String section, String key, String defaultValue)
    {

        String value = readItem(section, key);
        if (value == null)
        {
            return defaultValue;
        }
        else
        {
            return value;
        }
    }


    /**
     *  readInteger reads an int value of a key in a section. If the section or
     *  key is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue
     *@return               value of the key in this section
     */
    public int readInteger(String section, String key, int defaultValue)
    {

        String strValue = readItem(section, key);
        if (strValue == null)
        {
            return defaultValue;
        }
        else
        {
            return new Integer(strValue).intValue();
        }
    }


    /**
     *  readDouble reads a double value of a key in a section. If the section or
     *  key is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue
     *@return               value of the key in this section
     */
    public double readDouble(String section, String key, double defaultValue)
    {

        String strValue = readItem(section, key);
        if (strValue == null)
        {
            return defaultValue;
        }
        else
        {
            return new Double(strValue).doubleValue();
        }
    }


    /**
     *  readBool reads a boolean value of a key in a section. If the section or
     *  key is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue
     *@return               value of the key in this section
     */
    public boolean readBool(String section, String key, boolean defaultValue)
    {

        String strValue = readItem(section, key);
        if (strValue == null)
        {
            return defaultValue;
        }
        else
        {
            return (new Integer(strValue).intValue() != 0);
        }
    }


    /**
     *  read reads a string value of a key in a section. If the section or key
     *  is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue  Description of Parameter
     *@return               string value of the key in this section
     */
    public String read(String section, String key, String defaultValue)
    {
        return readString(section, key, defaultValue);
    }


    /**
     *  read reads an int value of a key in a section. If the section or key is
     *  unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue  Description of Parameter
     *@return               int value of the key in this section
     */
    public int read(String section, String key, int defaultValue)
    {
        return readInteger(section, key, defaultValue);
    }


    /**
     *  read reads a double value of a key in a section. If the section or key
     *  is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue  Description of Parameter
     *@return               double value of the key in this section
     */
    public double read(String section, String key, double defaultValue)
    {
        return readDouble(section, key, defaultValue);
    }


    /**
     *  read reads a boolean value of a key in a section. If the section or key
     *  is unknown then it returns the default value.
     *
     *@param  section
     *@param  key
     *@param  defaultValue  Description of Parameter
     *@return               boolean value of the key in this section
     */
    public boolean read(String section, String key, boolean defaultValue)
    {
        return readBool(section, key, defaultValue);
    }


    /**
     *  writeString writes a value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value
     */
    public void writeString(String section, String key, String value)
    {
        Properties props = getSectionValues(section);
        if (props == null)
        {
            props = new Properties();
            data.put(section, props);
        }
        props.setProperty(key, value);
    }


    /**
     *  writeInteger writes an int value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value
     */
    public void writeInteger(String section, String key, int value)
    {
        writeString(section, key, new Integer(value).toString());
    }


    /**
     *  writeDouble writes a double value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value
     */
    public void writeDouble(String section, String key, double value)
    {
        writeString(section, key, new Double(value).toString());
    }


    /**
     *  writeBool writes a boolean value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value
     */
    public void writeBool(String section, String key, boolean value)
    {
        int intValue;
        if (value)
        {
            intValue = 1;
        }
        else
        {
            intValue = 0;
        }
        writeString(section, key, new Integer(intValue).toString());
    }


    /**
     *  write writes a value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value    Description of Parameter
     */
    public void write(String section, String key, String value)
    {
        writeString(section, key, value);
    }


    /**
     *  write writes an int value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value    Description of Parameter
     */
    public void write(String section, String key, int value)
    {
        writeInteger(section, key, value);
    }


    /**
     *  write writes a double value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value    Description of Parameter
     */
    public void write(String section, String key, double value)
    {
        writeDouble(section, key, value);
    }


    /**
     *  write writes a boolean value of a key in a section.
     *
     *@param  section
     *@param  key
     *@param  value    Description of Parameter
     */
    public void write(String section, String key, boolean value)
    {
        writeBool(section, key, value);
    }


    /**
     *  Serialization of the object
     *
     *@return    whole object as string
     */

    public String toString()
    {
        try
        {
            StringWriter sWriter = new StringWriter();
            BufferedWriter writer = new BufferedWriter(sWriter);
            printOn(writer);
            writer.close();
            return sWriter.toString();
        }
        catch (IOException e)
        {
            return "IniFile (toString() failed)";
        }

    }


    /**
     *  Description of the Method
     *
     *@param  writer           Description of Parameter
     *@exception  IOException  Description of Exception
     */
    public void printOn(BufferedWriter writer) throws IOException
    {
        Iterator iter = data.entrySet().iterator();
        while (iter.hasNext())
        {
            Map.Entry entry = (Map.Entry) iter.next();
            writer.write("[" + entry.getKey() + "]");
            writer.newLine();
            Iterator iter1 = ((Map) entry.getValue()).entrySet().iterator();
            while (iter1.hasNext())
            {
                Map.Entry entry1 = (Map.Entry) iter1.next();
                writer.write(entry1.getKey() + "=" + entry1.getValue());
                writer.newLine();
            }
            writer.newLine();
        }

    }


    /**
     *  readItem reads a value of a key in a section.
     *
     *@param  section
     *@param  key
     *@return          value of the key in this section
     */
    private String readItem(String section, String key)
    {
        Properties props = getSectionValues(section);
        if (props == null)
        {
            return null;
        }
        return props.getProperty(key);
    }
}
